@@ -4,7 +4,9 @@ import os
 import requests
 import time
 
-
+from conda_smithy import vendored
+from conda_smithy.vendored import travis_encrypt as travis
+import ruamel.yaml
 # https://circleci.com/docs/api#add-environment-variable
 
 # curl -X POST --header "Content-Type: application/json" -d '{"name":"foo", "value":"bar"}'
@@ -79,7 +81,6 @@ def add_project_to_appveyor(user, project):
             response.raise_for_status()
         print(' * {}/{} has been enabled on appveyor'.format(user, project))
 
-    
 
 def appveyor_encrypt_binstar_token(feedstock_directory, user, project):
     import ruamel.yaml
@@ -147,9 +148,10 @@ def add_project_to_travis(user, project):
                 response = requests.post(synch_url, headers=headers)
             time.sleep(3)
 
-        if count > 5:
-            print('  * Unable to register the repo on Travis (is it down?)')
-            return
+        if count > 20:
+            msg = ('Unable to register the repo on Travis\n'
+                   '(Is it down? Is the "{}" name spelt correctly? [note: case sensitive])')
+            raise RuntimeError(msg.format(user))
 
     if found[0]['active'] is True:
         print(' * {}/{} already enabled on travis-ci'.format(user, project))
@@ -166,11 +168,8 @@ def add_project_to_travis(user, project):
 
 
 def travis_token_update_conda_forge_config(feedstock_directory, user, project):
-    import vendored
-    import vendored.travis_encrypt as travis
-    import ruamel.yaml
     item = 'BINSTAR_TOKEN="{}"'.format(os.environ['BINSTAR_TOKEN'])
-    slug =  "{}/{}".format(user, project)
+    slug = "{}/{}".format(user, project)
 
     forge_yaml = os.path.join(feedstock_directory, 'conda-forge.yml')
     if os.path.exists(forge_yaml):
@@ -183,11 +182,16 @@ def travis_token_update_conda_forge_config(feedstock_directory, user, project):
     if not code:
         code = {}
 
-    code.setdefault('travis', {}).setdefault('secure', {})['BINSTAR_TOKEN'] = travis.encrypt(slug, item)
+    code.setdefault('travis', {}).setdefault('secure', {})['BINSTAR_TOKEN'] = (
+        _encrypt_binstar_token(slug, item)
+    )
     with open(forge_yaml, 'w') as fh:
         fh.write(ruamel.yaml.dump(code, Dumper=ruamel.yaml.RoundTripDumper))
 
 
+def _encrypt_binstar_token(slug, item):
+    return travis.encrypt(slug, item.encode()).decode('utf-8')
+
 if __name__ == '__main__':
     import argparse
     parser = argparse.ArgumentParser()
